@@ -18,7 +18,7 @@ require_once 'Output.php';
 require_once 'CGE_ComodoCredentialException.php';
 require_once 'confusa_handler.php';
 require_once 'Translator.php';
-
+require_once 'Input.php';
 /* global config */
 require_once 'Config.php';
 require_once 'CA_Comodo.php';
@@ -227,13 +227,27 @@ class Framework {
 			$this->renderError = true;
 		}
 
+		if ($this->isCSRFAttempt()) {
+			Framework::error_output($this->contentPage->translateMessageTag('fw_anticsrf_msg'));
+			$this->tpl->assign('instance', Config::get_config('system_name'));
+			$this->tpl->assign('errors', self::$errors);
+			$this->tpl->display('site.tpl');
+			exit(0);
+		}
+
+		/* Create a new anti CSRF token and export to the template engine */
+		$this->current_anticsrf = self::getAntiCSRF();
+		$this->tpl->assign('ganticsrf', 'anticsrf='.$this->current_anticsrf);
+		$this->tpl->assign('panticsrf',
+				   '<input type="hidden" name="anticsrf" value="'.
+				   $this->current_anticsrf.'" />');
 		/*
 		 * Try to run the pre-processing
 		 */
 		try {
 			$res = $this->contentPage->pre_process($this->person);
 			if ($res) {
-				$this->tpl->assign('extraHeader', $res);
+				$this->tpl->assign('extraHeader');
 			}
 		} catch (CGE_RemoteCredentialException $rce) {
 			$msg  = $this->contentPage->translateMessageTag('fw_error_remote_credential1');
@@ -346,7 +360,9 @@ class Framework {
 		}
 		$this->tpl->display('site.tpl');
 
-		$this->contentPage->post_process($this->person);
+		if (!$this->renderError) {
+			$this->contentPage->post_process($this->person);
+		}
 	} /* end start() */
 
 	/**
@@ -394,6 +410,81 @@ class Framework {
 		}
 	} /* end applyNRENBranding */
 
+	/**
+	 * isCSRFAttempt() - test to see if the current connection is an CSRF
+	 *
+	 * If a GET or POST is set, the client tries to send data to the
+	 * portal, and we must make sure that the user is not being fooled by a
+	 * malicious site.
+	 *
+	 * @param	void
+	 * @return	boolean status indicating if a detectable
+	 * @access	private
+	 */
+	private function isCSRFAttempt()
+	{
+		if (!empty($_GET) || !empty($_POST)) {
+			$facsrft = null;
+			if (isset($_GET) && array_key_exists('anticsrf', $_GET)) {
+				$facsrft = Input::sanitizeAntiCSRFToken($_GET['anticsrf']);
+			} else if (isset($_POST) && array_key_exists('anticsrf', $_POST)) {
+				$facsrft = Input::sanitizeAntiCSRFToken($_POST['anticsrf']);
+			}
+
+			if (!self::validateACSRFT($facsrft)) {
+				$msg =  "Got a GET/POST request without the correct anticsrf tag.";
+				if (array_key_exists('HTTP_REFERER', $_SERVER)) {
+					$msg .= " Referer was " . $_SERVER['HTTP_REFERER'];
+				}
+				Logger::log_event(LOG_WARNING, "[Anti CSRF] $msg");
+				return true;
+			}
+
+			/* If start_login is set, referer will be another server
+			 * (the IdP), so stop testing here */
+			if (array_key_exists('start_login', $_GET)) {
+				if (Input::sanitizeText($_GET['start_login']) == "yes") {
+					return false;
+				}
+			}
+
+			/* Not a direct CSRF, but verify the HTTP_REFERER to
+			 * avoid replay attacks of the token if it is somehow
+			 * leaked
+			 */
+			if (array_key_exists('HTTP_REFERER', $_SERVER)) {
+				$url  = Input::sanitizeURL("http" . (($_SERVER['HTTPS'] == 'on') ? "s" : "").
+							   "://" . $_SERVER['HTTP_HOST'] .
+							   $_SERVER['SCRIPT_NAME']);
+				$rurl = Input::sanitizeURL(Input::sanitizeURL($_SERVER['HTTP_REFERER']));
+
+				/* Strip out all GET-params in the URL */
+				$p = strpos($rurl, "?");
+				if ($p) {
+					$rurl = substr($rurl, 0, $p);
+				}
+				if (dirname($url) !== dirname($rurl)) {
+					Logger::log_event(LOG_ALERT, "[Anti CSRF] Got correct anti-csrf from client, but HTTP_REFERER was set." .
+							  "Got $rurl, expected $url. Possible CSRF attempt. Request was blocked.");
+					return true;
+				}
+			}
+		}
+		return false; /* no detectable CSRF attempt */
+	} /* end CSRFAttempt() */
+
+	/**
+	 * Register new error-message to display in the page
+	 *
+	 * This will post a message in the top of the general content-area
+	 * surrounded with a red (if the CSS is unaltered) box to draw
+	 * attention.
+	 *
+	 * @param	String $message the error to display
+	 * @return	void
+	 * @access	public
+	 * @
+	 */
 	public static function error_output($message)
 	{
 		self::$errors[] = $message;
@@ -412,4 +503,61 @@ class Framework {
 	{
 		self::$warnings[] = $message;
 	}
+
+	/**
+	 * getAntiCSRF() return a per-session unique identifier
+	 *
+	 * We could introduce the session_id, but to aovid session-hijacking we
+	 * do not want to include this into all the form-requests in Confusa.
+	 *
+	 * Instead we use a second-order identifier derived from the session
+	 * along with extra randomness and the current time. This is then
+	 * concatenated and fed through a one-way function (to proect the
+	 * session_id).
+	 *
+	 * Why not rely upon HTTP_REFERER as a test?
+	 * - *nothing* from a browser (with respect to anti CSRF) can be trusted.
+	 * - it is easy to fake a HTTP_REFERER value by explicitly setting the
+	 *   headers sent.
+	 * - browsers are vulnerable, exploits exists
+	 *
+	 * @param	String $rand	A random seed. If not supplied, a random
+	 *				value is generated.
+	 * @return	String		The Anti CSRF token.
+	 * @access	static
+	 */
+	public static function getAntiCSRF($rand = null)
+	{
+		if (is_null($rand)) {
+			$rand = rand(0, PHP_INT_MAX);
+		}
+		/* make sure rand only contains allowed characters. */
+		$rand = Input::sanitizeAntiCSRFToken($rand);
+
+		return $rand.":".sha1(session_id().$rand);
+	} /* end getAntiCSRF() */
+
+
+	/**
+	 * validateACSRFT() validate a supplied token
+	 *
+	 * The function takes an arbitrary token and tries to generate a
+	 * matching token from the salt. If successful, the token is valid.
+	 *
+	 * @param	String	$token	The token to validate
+	 * @return	Boolean		True if the token is valid
+	 * @access	static
+	 */
+	public static function validateACSRFT($token)
+	{
+		if (is_null($token) || $token == "") {
+			return false;
+		}
+		$pos = strrpos($token, ":");
+		if (!$pos) {
+			throw new ConfusaGenException("Malformed Anti CSRF token, could not determine placement of delimiter.");
+		}
+		return $token === self::getAntiCSRF(substr($token, 0, $pos));
+	} /* end validateACSRFT() */
+
 } /* end class Framework */
