@@ -86,7 +86,7 @@ final class CP_ProcessCsr extends Content_Page
 			/* assign the order_number again */
 			$this->tpl->assign('order_number', $order_number);
 			$this->tpl->assign('status_poll', true);
-
+			$this->tpl->assign('ganticsrf', "anticsrf=".Input::sanitizeAntiCSRFToken($_GET['anticsrf']));
 			if ($this->ca->pollCertStatus($order_number)) {
 			    $this->tpl->assign('done', TRUE);
 			}
@@ -474,9 +474,11 @@ final class CP_ProcessCsr extends Content_Page
 		$url = "http";
 		if ($_SERVER['SERVER_PORT'] == 443)
 			$url .= "s";
-		$url .= "://" . $_SERVER['HTTP_HOST'] . "/" . dirname($_SERVER['PHP_SELF']) . "/download_certificate.php?poll=$authToken";
+		$url .= "://" . $_SERVER['HTTP_HOST'] . "/" . dirname($_SERVER['PHP_SELF']) .
+			"/download_certificate.php?poll=$authToken" .
+			"&amp;anticsrf=" . Framework::getAntiCSRF();
 		return "<META HTTP-EQUIV=\"REFRESH\" content=\"3; url=$url\">\n";
-	} /* end approve_csr_remote() */
+	} /* end approveCSR() */
 
 
 	/**
